@@ -232,7 +232,8 @@ class Up2k(object):
                 (ft if v is True else ff if v is False else fv).format(k, str(v))
                 for k, v in flags.items()
             ]
-            self.log(" ".join(sorted(a)) + "\033[0m")
+            if a:
+                self.log(" ".join(sorted(a)) + "\033[0m")
 
             reg = {}
             path = os.path.join(ptop, ".hist", "up2k.snap")
@@ -288,9 +289,9 @@ class Up2k(object):
         self.pp.n = next(dbw[0].execute("select count(w) from up"))[0]
 
         excl = [
-            vol.realpath + d.vpath[len(vol.vpath) :]
+            vol.realpath + "/" + d.vpath[len(vol.vpath) :].lstrip("/")
             for d in all_vols
-            if d.vpath.startswith(vol.vpath + "/")
+            if d != vol and (d.vpath.startswith(vol.vpath + "/") or not vol.vpath)
         ]
         n_add = self._build_dir(dbw, top, set(excl), top)
         n_rm = self._drop_lost(dbw[0], top)
@@ -1309,6 +1310,7 @@ class Up2k(object):
                     self.log("no cursor to write tags with??", c=1)
                     continue
 
+                # TODO is undef if vol 404 on startup
                 entags = self.entags[ptop]
                 if not entags:
                     self.log("no entags okay.jpg", c=3)
