@@ -753,7 +753,7 @@ document.onkeydown = function (e) {
 		clearTimeout(search_timeout);
 		var now = new Date().getTime();
 		if (now - search_in_progress > 30 * 1000)
-			search_timeout = setTimeout(do_search, 100);
+			search_timeout = setTimeout(do_search, 200);
 	}
 
 	function do_search() {
@@ -796,6 +796,8 @@ document.onkeydown = function (e) {
 		var res = JSON.parse(this.responseText),
 			tagord = res.tag_order;
 
+		sortfiles(res.hits);
+
 		var ofiles = ebi('files');
 		if (ofiles.getAttribute('ts') > this.ts)
 			return;
@@ -814,7 +816,7 @@ document.onkeydown = function (e) {
 
 		var html = mk_files_header(tagord);
 		html.push('<tbody>');
-		html.push('<tr><td>-</td><td colspan="42"><a href="#" id="unsearch">close search results</a></td></tr>');
+		html.push('<tr><td>-</td><td colspan="42"><a href="#" id="unsearch">! close search results</a></td></tr>');
 		for (var a = 0; a < res.hits.length; a++) {
 			var r = res.hits[a],
 				ts = parseInt(r.ts),
@@ -833,7 +835,7 @@ document.onkeydown = function (e) {
 					v = r.tags[k] || "";
 
 				if (k == ".dur") {
-					var sv = s2ms(v);
+					var sv = v ? s2ms(v) : "";
 					nodes[nodes.length - 1] += '</td><td sortv="' + v + '">' + sv;
 					continue;
 				}
@@ -1085,32 +1087,8 @@ var treectl = (function () {
 		}
 
 		ebi('srv_info').innerHTML = '<span>' + res.srvinf + '</span>';
-		var nodes = res.dirs.concat(res.files),
-			sopts = jread('fsort', []);
-
-		try {
-			for (var a = sopts.length - 1; a >= 0; a--) {
-				var name = sopts[a][0], rev = sopts[a][1], typ = sopts[a][2];
-				if (name.indexOf('tags/') == -1) {
-					nodes.sort(function (v1, v2) {
-						if (!v1[name]) return -1 * rev;
-						if (!v2[name]) return 1 * rev;
-						return rev * (typ == 'int' ? (v1[name] - v2[name]) : (v1[name].localeCompare(v2[name])));
-					});
-				}
-				else {
-					name = name.slice(5);
-					nodes.sort(function (v1, v2) {
-						if (!v1.tags[name]) return -1 * rev;
-						if (!v2.tags[name]) return 1 * rev;
-						return rev * (typ == 'int' ? (v1.tags[name] - v2.tags[name]) : (v1.tags[name].localeCompare(v2.tags[name])));
-					});
-				}
-			}
-		}
-		catch (ex) {
-			console.log("failed to apply sort config: " + ex);
-		}
+		var nodes = res.dirs.concat(res.files);
+		nodes = sortfiles(nodes);
 
 		var top = this.top;
 		var html = mk_files_header(res.taglist);
@@ -1125,7 +1103,7 @@ var treectl = (function () {
 					v = (r.tags || {})[k] || "";
 
 				if (k == ".dur") {
-					var sv = s2ms(v);
+					var sv = v ? s2ms(v) : "";
 					ln[ln.length - 1] += '</td><td sortv="' + v + '">' + sv;
 					continue;
 				}
@@ -1149,6 +1127,7 @@ var treectl = (function () {
 
 		filecols.set_style();
 		mukey.render();
+		msel.render();
 		reload_tree();
 		reload_browser();
 	}
@@ -1304,7 +1283,7 @@ function find_file_col(txt) {
 function mk_files_header(taglist) {
 	var html = [
 		'<thead>',
-		'<th></th>',
+		'<th name="lead"><span>c</span></th>',
 		'<th name="href"><span>File Name</span></th>',
 		'<th name="sz" sort="int"><span>Size</span></th>'
 	];
@@ -1407,8 +1386,8 @@ var filecols = (function () {
 		if (!min)
 			for (var a = 0, aa = rows.length; a < aa; a++) {
 				var c = rows[a].cells[i];
-				if (c)
-					var v = c.textContent = s2ms(c.textContent);
+				if (c && c.textContent)
+					c.textContent = s2ms(c.textContent);
 			}
 	}
 	catch (ex) { }
@@ -1479,8 +1458,11 @@ var mukey = (function () {
 	}
 
 	function render() {
-		var ci = find_file_col('Key'),
-			i = ci[0],
+		var ci = find_file_col('Key');
+		if (!ci)
+			return;
+
+		var i = ci[0],
 			min = ci[1],
 			rows = ebi('files').tBodies[0].rows;
 
@@ -1529,7 +1511,10 @@ var mukey = (function () {
 
 
 function addcrc() {
-	var links = document.querySelectorAll('#files>tbody>tr>td:nth-child(2)>a');
+	var links = document.querySelectorAll(
+		'#files>tbody>tr>td:nth-child(2)>' + (
+			ebi('unsearch') ? 'div>a:last-child' : 'a'));
+
 	for (var a = 0, aa = links.length; a < aa; a++)
 		if (!links[a].getAttribute('id'))
 			links[a].setAttribute('id', 'f-' + crc32(links[a].textContent));
@@ -1559,6 +1544,141 @@ function addcrc() {
 })();
 
 
+var arcfmt = (function () {
+	if (!ebi('arc_fmt'))
+		return { "render": function () { } };
+
+	var html = [],
+		arcfmts = ["tar", "zip", "zip_dos", "zip_crc"],
+		arcv = ["tar", "zip=utf8", "zip", "zip=crc"];
+
+	for (var a = 0; a < arcfmts.length; a++) {
+		var k = arcfmts[a];
+		html.push(
+			'<span><input type="radio" name="arcfmt" value="' + k + '" id="arcfmt_' + k + '">' +
+			'<label for="arcfmt_' + k + '">' + k + '</label></span>');
+	}
+	ebi('arc_fmt').innerHTML = html.join('\n');
+
+	var fmt = sread("arc_fmt") || "zip";
+	ebi('arcfmt_' + fmt).checked = true;
+
+	function render() {
+		var arg = arcv[arcfmts.indexOf(fmt)],
+			tds = document.querySelectorAll('#files tbody td:first-child a');
+
+		for (var a = 0, aa = tds.length; a < aa; a++) {
+			var o = tds[a], txt = o.textContent, href = o.getAttribute('href');
+			if (txt != 'tar' && txt != 'zip')
+				continue;
+
+			var ofs = href.lastIndexOf('?');
+			if (ofs < 0)
+				throw 'missing arg in url';
+
+			o.setAttribute("href", href.slice(0, ofs + 1) + arg);
+			o.textContent = fmt.split('_')[0];
+		}
+		ebi('selzip').textContent = fmt.split('_')[0];
+		ebi('selzip').setAttribute('fmt', arg);
+	}
+
+	function try_render() {
+		try {
+			render();
+		}
+		catch (ex) {
+			console.log("arcfmt failed: " + ex);
+		}
+	}
+
+	function change_fmt(e) {
+		ev(e);
+		fmt = this.getAttribute('value');
+		swrite("arc_fmt", fmt);
+		try_render();
+	}
+
+	var o = document.querySelectorAll('#arc_fmt input');
+	for (var a = 0; a < o.length; a++) {
+		o[a].onchange = change_fmt;
+	}
+
+	return {
+		"render": try_render
+	};
+})();
+
+
+var msel = (function () {
+	function getsel() {
+		var names = [];
+		var links = document.querySelectorAll('#files tbody tr.sel td:nth-child(2) a');
+		for (var a = 0, aa = links.length; a < aa; a++)
+			names.push(links[a].getAttribute('href').replace(/\/$/, "").split('/').slice(-1));
+
+		return names;
+	}
+	function selui() {
+		var fun = getsel().length ? "add" : "remove";
+		ebi('wtoggle').classList[fun]('sel');
+	}
+	function seltgl(e) {
+		ev(e);
+		var tr = this.parentNode;
+		tr.classList.toggle('sel');
+		selui();
+	}
+	function evsel(e, fun) {
+		ev(e);
+		var trs = document.querySelectorAll('#files tbody tr');
+		for (var a = 0, aa = trs.length; a < aa; a++)
+			trs[a].classList[fun]('sel');
+		selui();
+	}
+	ebi('selall').onclick = function (e) {
+		evsel(e, "add");
+	};
+	ebi('selinv').onclick = function (e) {
+		evsel(e, "toggle");
+	};
+	ebi('selzip').onclick = function (e) {
+		ev(e);
+		var names = getsel();
+		var arg = ebi('selzip').getAttribute('fmt');
+		var txt = names.join('\n');
+		var frm = document.createElement('form');
+		frm.setAttribute('action', '?' + arg);
+		frm.setAttribute('method', 'post');
+		frm.setAttribute('target', '_blank');
+		frm.setAttribute('enctype', 'multipart/form-data');
+		frm.innerHTML = '<input name="act" value="zip" />' +
+			'<textarea name="files" id="ziptxt"></textarea>';
+		frm.style.display = 'none';
+
+		var oldform = document.querySelector('#widgeti>form');
+		if (oldform)
+			oldform.parentNode.removeChild(oldform);
+
+		ebi('widgeti').appendChild(frm);
+		var obj = ebi('ziptxt');
+		obj.value = txt;
+		console.log(txt);
+		frm.submit();
+	};
+	function render() {
+		var tds = document.querySelectorAll('#files tbody td+td+td');
+		for (var a = 0, aa = tds.length; a < aa; a++) {
+			tds[a].onclick = seltgl;
+		}
+		arcfmt.render();
+	}
+	return {
+		"render": render
+	};
+})();
+
+
 function ev_row_tgl(e) {
 	ev(e);
 	filecols.toggle(this.parentElement.parentElement.getElementsByTagName('span')[0].textContent);
@@ -1611,3 +1731,4 @@ function reload_browser(not_mp) {
 }
 reload_browser(true);
 mukey.render();
+msel.render();
